@@ -6,6 +6,7 @@ pub struct Config {
     pub(crate) arguments: Vec<OsString>,
     pub(crate) log_command: bool,
     pub(crate) working_directory: Option<PathBuf>,
+    pub(crate) added_environment_variables: Vec<(OsString, OsString)>,
     pub(crate) stdin: Arc<Vec<u8>>,
     pub(crate) relay_stdout: bool,
     pub(crate) relay_stderr: bool,
@@ -39,6 +40,7 @@ impl Default for Config {
             arguments: Vec::new(),
             log_command: false,
             working_directory: None,
+            added_environment_variables: Vec::new(),
             stdin: Arc::new(Vec::new()),
             relay_stdout: true,
             relay_stderr: true,
