@@ -25,24 +25,21 @@ use std::{
 /// Here's a non-exhaustive list of the most commonly used types to get you started:
 ///
 /// - [`String`] and [`&str`],
-/// - [`Split`] (and its shortcut `%`),
+/// - [`Split`] (and its shortcut `%`) to split commands by whitespace,
 /// - [`PathBuf`] and [`&Path`],
 /// - multiple sequence types, like [`vectors`], [`slices`] and (since version 1.51) [`arrays`],
 /// - [`CurrentDir`],
-/// - [`StdIn`], and
+/// - [`Env`] for setting environment variables,
+/// - [`Stdin`], and
 /// - [`LogCommand`].
 ///
 /// [`String`]: trait.Input.html#impl-Input-for-String
 /// [`&str`]: trait.Input.html#impl-Input-for-%26str
-/// [`Split`]: trait.Input.html#impl-Input-3
 /// [`PathBuf`]: trait.Input.html#impl-Input-for-PathBuf
 /// [`&Path`]: trait.Input.html#impl-Input-for-%26Path
 /// [`vectors`]: trait.Input.html#impl-Input-for-Vec<T>
 /// [`slices`]: trait.Input.html#impl-Input-for-%26[T]
 /// [`arrays`]: trait.Input.html#impl-Input-for-[T%3B%20N]
-/// [`CurrentDir`]: trait.Input.html#impl-Input-1
-/// [`StdIn`]: trait.Input.html#impl-Input-2
-/// [`LogCommand`]: trait.Input.html#impl-Input
 ///
 /// ## Tuples
 ///
@@ -171,9 +168,6 @@ impl Input for String {
     }
 }
 
-/// See the [`Input`] implementation for [`Split`] below.
-pub struct Split<T: AsRef<str>>(pub T);
-
 /// Splits the contained string by whitespace (using [`split_whitespace`])
 /// and uses the resulting words as separate arguments.
 ///
@@ -198,7 +192,9 @@ pub struct Split<T: AsRef<str>>(pub T);
 /// ```
 ///
 /// [`split_whitespace`]: str::split_whitespace
-impl<T: AsRef<str>> Input for Split<T> {
+pub struct Split<T: AsRef<str>>(pub T);
+
+impl<T: AsRef<str>> Input for crate::input::Split<T> {
     #[doc(hidden)]
     fn configure(self, config: &mut Config) {
         for argument in self.0.as_ref().split_whitespace() {
@@ -266,6 +262,33 @@ impl<'a> Input for std::str::SplitAsciiWhitespace<'a> {
     }
 }
 
+impl Input for () {
+    #[doc(hidden)]
+    fn configure(self, _: &mut Config) {}
+}
+
+macro_rules! tuple_impl {
+    ($($index:tt, $generics:ident,)+) => {
+        impl<$($generics),+> Input for ($($generics,)+)
+        where
+            $($generics: Input,)+
+        {
+            #[doc(hidden)]
+            fn configure(self, config: &mut Config) {
+                $(<$generics as Input>::configure(self.$index, config);)+
+            }
+        }
+    };
+}
+
+tuple_impl!(0, A,);
+tuple_impl!(0, A, 1, B,);
+tuple_impl!(0, A, 1, B, 2, C,);
+tuple_impl!(0, A, 1, B, 2, C, 3, D,);
+tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E,);
+tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E, 5, F,);
+tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E, 5, F, 6, G,);
+
 /// All elements of the given [`Vec`] are used as arguments to [`cmd!`].
 /// Same as passing in the elements separately.
 ///
@@ -323,37 +346,6 @@ where
     }
 }
 
-impl Input for () {
-    #[doc(hidden)]
-    fn configure(self, _: &mut Config) {}
-}
-
-macro_rules! tuple_impl {
-    ($($index:tt, $generics:ident,)+) => {
-        impl<$($generics),+> Input for ($($generics,)+)
-        where
-            $($generics: Input,)+
-        {
-            #[doc(hidden)]
-            fn configure(self, config: &mut Config) {
-                $(<$generics as Input>::configure(self.$index, config);)+
-            }
-        }
-    };
-}
-
-tuple_impl!(0, A,);
-tuple_impl!(0, A, 1, B,);
-tuple_impl!(0, A, 1, B, 2, C,);
-tuple_impl!(0, A, 1, B, 2, C, 3, D,);
-tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E,);
-tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E, 5, F,);
-tuple_impl!(0, A, 1, B, 2, C, 3, D, 4, E, 5, F, 6, G,);
-
-/// See the [`Input`] implementation for [`LogCommand`] below.
-#[derive(Clone, Debug)]
-pub struct LogCommand;
-
 /// Passing in [`LogCommand`] as an argument to [`cmd!`] will cause it
 /// to log the commands (including all arguments) to `stderr`.
 /// (This is similar `bash`'s `-x` option.)
@@ -364,6 +356,9 @@ pub struct LogCommand;
 /// cmd_unit!(LogCommand, %"echo foo");
 /// // writes '+ echo foo' to stderr
 /// ```
+#[derive(Clone, Debug)]
+pub struct LogCommand;
+
 impl Input for LogCommand {
     #[doc(hidden)]
     fn configure(self, config: &mut Config) {
@@ -371,9 +366,6 @@ impl Input for LogCommand {
     }
 }
 
-/// See the [`Input`] implementation for [`CurrentDir`] below.
-pub struct CurrentDir<T: AsRef<Path>>(pub T);
-
 /// By default child processes inherit the current directory from their
 /// parent. You can override this with [`CurrentDir`]:
 ///
@@ -388,6 +380,8 @@ pub struct CurrentDir<T: AsRef<Path>>(pub T);
 /// ```
 ///
 /// Paths that are relative to the parent's current directory are allowed.
+pub struct CurrentDir<T: AsRef<Path>>(pub T);
+
 impl<T> Input for CurrentDir<T>
 where
     T: AsRef<Path>,
@@ -434,9 +428,6 @@ impl Input for &Path {
     }
 }
 
-/// See the [`Input`] implementation for [`Stdin`] below.
-pub struct Stdin<T: AsRef<[u8]>>(pub T);
-
 /// Writes the given byte slice to the child's standard input.
 ///
 /// ```
@@ -451,6 +442,8 @@ pub struct Stdin<T: AsRef<[u8]>>(pub T);
 ///
 /// If `Stdin` is used multiple times, all given bytes slices will be written
 /// to the child's standard input in order.
+pub struct Stdin<T: AsRef<[u8]>>(pub T);
+
 impl<T> Input for Stdin<T>
 where
     T: AsRef<[u8]>,
@@ -460,3 +453,35 @@ where
         Arc::make_mut(&mut config.stdin).extend_from_slice(self.0.as_ref());
     }
 }
+
+/// Adds an environment variable to the environment of the child process.
+///
+/// ```
+/// use cradle::*;
+///
+/// let StdoutUntrimmed(output) = cmd!("env", Env("FOO", "bar"));
+/// assert!(output.contains("FOO=bar\n"));
+/// ```
+///
+/// Child processes inherit the environment of the parent process.
+/// [`Env`] only adds environment variables to that inherited environment.
+/// If the environment variable is also set in the parent process,
+/// it is overwritten by [`Env`].
+pub struct Env<Key, Value>(pub Key, pub Value)
+where
+    Key: AsRef<OsStr>,
+    Value: AsRef<OsStr>;
+
+impl<Key, Value> Input for Env<Key, Value>
+where
+    Key: AsRef<OsStr>,
+    Value: AsRef<OsStr>,
+{
+    #[doc(hidden)]
+    fn configure(self, config: &mut Config) {
+        let Self(key, value) = self;
+        config
+            .added_environment_variables
+            .push((key.as_ref().to_os_string(), value.as_ref().to_os_string()));
+    }
+}
